/*
 *    TArtMapper.cc
 *
 *    by T. Isobe
 */

#include "TArtMapper.h"
#include "TArtConfigFile.h"
TArtMapper* TArtMapper::sMapper = 0;
//////////////////////////////////////////////////

TArtMapper::TArtMapper()
{
   TArtCore::Info("TArtMapper","Constructor");
}

bool TArtMapper::isRecoLoaded()
{
  return recomap.size()>0 ? true : false;
}

bool TArtMapper::isLoaded()
{
  return anamap.size()>0 ? true : false;
}

TArtMapper::~TArtMapper()
{
}

TArtMapper* TArtMapper::Instance() {
  if (!sMapper) sMapper = new TArtMapper();                      
  return sMapper;   
}

unsigned int *TArtMapper::FindRecoMap(int fp,int detid,int geo,int ch)
{
  TArtRIDFMap mm(fp,detid,geo,ch);
  return FindRecoMap(&mm);
}
TArtMap *TArtMapper::FindMap(int fp,int detid,int geo,int ch)
{
  TArtRIDFMap mm(fp,detid,geo,ch);
  return FindMap(&mm);
}

unsigned int *TArtMapper::FindRecoMap(TArtRIDFMap *inmap)
{

  rmap = recomap.find(*inmap);

  if(rmap != ritrEnd){
    return rmap->second;
  }

  return NULL;
}
TArtMap *TArtMapper::FindMap(TArtRIDFMap *inmap)
{

  amap = anamap.find(*inmap);

  if(amap != itrEnd){
    return &(amap->second);
  }

  return NULL;
}


void TArtMapper::LoadRecoConfig(TArtRIDFMap *map, unsigned int *p){
  recomap.insert(std::pair<TArtRIDFMap, unsigned int *>(*map, p));
  ritrEnd = recomap.end();
  return;
}

void TArtMapper::LoadRecoConfig(int fp,int detid,int geo,int ch,unsigned int *p){
  TArtRIDFMap map(fp,detid,geo,ch);
  LoadRecoConfig(&map,p);
  return;
}

void TArtMapper::LoadConfig(const char* filename){

  TArtCore::Info("TArtMapper","LoadConfig file = '%s'",filename);
  TArtConfigFile configFile(filename);
  const char *str;
  const char *mapname = 0;
  int ndata;
  while (1) {
    if (!(mapname = configFile.GetNextToken())) break;
    if (!configFile.GetNextTokenAsInt(ndata)) break;
    TArtCore::Info("TArtMapper","LoadConfig file = '%s'",mapname);
    TArtCore::Info("TArtMapper","LoadConfig ndata = %d",ndata);
    TArtConfigFile mapFile(mapname);
    while (1) {
      int catid, detid;
      const int nids = 5;
      int ids[nids];
      if (!mapFile.GetNextTokenAsInt(catid)) break;
      if (!mapFile.GetNextTokenAsInt(detid)) break;
      for (int i=0; i!=ndata; i++) {
	for (int j=0; j!=nids; j++) {
	  if (!mapFile.GetNextTokenAsInt(ids[j])) break;
	}
	// 2008/12/09 23:20 S.Takeuchi for check 
	printf("%s: %d,%d,%d,%d,%d is mapped to %d, %d,%d\n",mapname,ids[0],ids[1],ids[2],ids[3],ids[4],catid,detid,i+1);
	//
	TArtMap map1(catid,detid,i+1);
	// ids[0]:device is not used
	TArtRIDFMap map2(ids[1],ids[2],ids[3],ids[4]);
	anamap.insert(std::pair<TArtRIDFMap, TArtMap>(map2, map1));
      }
    }
  }

  itrEnd = anamap.end();

  TArtCore::Info("TArtMapper","LoadConfig loaded");
  return;
}
