
#include "TArtSimpleFunction.hh"
#include "TArtHistManager.hh"
#include "TArtCanvasManager.hh"

#include "TH1.h"
#include "TH1D.h"
#include "TH2.h"
#include "TProfile.h"
#include "TFitResult.h"
#include "TFitResultPtr.h"
#include "TDirectory.h"
#include "TList.h"

#include <iostream>
#include <iomanip>
#include <string>
#include <sstream>

TFitResultPtr TArtSimpleFunction::Fit(const char* formula, Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax)
{
  TH1* hist = gTH1;
  if(hist){
    return hist->Fit(formula, option, goption, xmin, xmax);
  }else{
   return TFitResultPtr();
  }
}

TFitResultPtr TArtSimpleFunction::FitPolynomial(int n, Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax)
{
  std::ostringstream os;
  os << n;
  std::string pol("pol" + os.str());

  return Fit(pol.c_str(), option, goption, xmin, xmax);
}

TFitResultPtr TArtSimpleFunction::FitExponential(Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax)
{
  return Fit("expo", option, goption, xmin, xmax);
}

TFitResultPtr TArtSimpleFunction::FitGaussian(Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax)
{
  return Fit("gaus", option, goption, xmin, xmax);
}

TFitResultPtr TArtSimpleFunction::FitLandau(Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax)
{
  return Fit("landau", option, goption, xmin, xmax);
}

TProfile* TArtSimpleFunction::ProfileX(Int_t firstybin, Int_t lastybin, Option_t* option)
{
  TH1* hist = gTH1;
  if(!hist) return 0;
  if(!hist->InheritsFrom("TH2")){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = (TH2*)hist;
  std::string name(hist2->GetName());
  name += "_pfx";
  TProfile* prof = hist2->ProfileX(name.c_str(), firstybin, lastybin, option);
  prof->SetMinimum(hist2->GetYaxis()->GetXmin());
  prof->SetMaximum(hist2->GetYaxis()->GetXmax());
  prof->ResetAttFill();
  prof->SetOption("E");

  TList* list = gDirectory->GetList();
  if(list){
    prof->SetDirectory(0);
    list->AddAfter(hist2, prof);
  }

  TArtCanvasManager::cdnext();
  TArtHistManager::DrawHist(prof);

  return prof;
}

TProfile* TArtSimpleFunction::ProfileY(Int_t firstxbin, Int_t lastxbin, Option_t* option)
{
  TH1* hist = gTH1;
  if(!hist) return 0;
  if(!hist->InheritsFrom("TH2")){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = (TH2*)hist;
  std::string name(hist2->GetName());
  name += "_pfy";
  TProfile* prof = hist2->ProfileY(name.c_str(), firstxbin, lastxbin, option);
  prof->SetMinimum(hist2->GetXaxis()->GetXmin());
  prof->SetMaximum(hist2->GetXaxis()->GetXmax());
  prof->ResetAttFill();
  prof->SetOption("E");

  TList* list = gDirectory->GetList();
  if(list){
    prof->SetDirectory(0);
    list->AddAfter(hist2, prof);
  }

  TArtCanvasManager::cdnext();
  TArtHistManager::DrawHist(prof);

  return prof;
}

TH1D* TArtSimpleFunction::ProjectionX(Int_t firstybin, Int_t lastybin, Option_t* option)
{
  TH1* hist = gTH1;
  if(!hist) return 0;
  if(!hist->InheritsFrom("TH2")){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = (TH2*)hist;
  std::string name(hist2->GetName());
  name += "_px";
  TH1D* histd = hist2->ProjectionX(name.c_str(), firstybin, lastybin, option);

  TList* list = gDirectory->GetList();
  if(list){
    histd->SetDirectory(0);
    list->AddAfter(hist2, histd);
  }

  TArtCanvasManager::cdnext();
  TArtHistManager::DrawHist(histd);

  return histd;
}

TH1D* TArtSimpleFunction::ProjectionY(Int_t firstxbin, Int_t lastxbin, Option_t* option)
{
  TH1* hist = gTH1;
  if(!hist) return 0;
  if(!hist->InheritsFrom("TH2")){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = (TH2*)hist;
  std::string name(hist2->GetName());
  name += "_py";
  TH1D* histd = hist2->ProjectionY(name.c_str(), firstxbin, lastxbin, option);

  TList* list = gDirectory->GetList();
  if(list){
    histd->SetDirectory(0);
    list->AddAfter(hist2, histd);
  }

  TArtCanvasManager::cdnext();
  TArtHistManager::DrawHist(histd);

  return histd;
}

TH1D* TArtSimpleFunction::BanY(double firsty, double lasty, Option_t* option)
{
  TH1* hist = gTH1;
  if(!hist) return 0;
  if(!hist->InheritsFrom("TH2")){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = (TH2*)hist;
  std::string name(hist2->GetName());
  name += "_bny";
  Int_t firstybin = hist2->GetYaxis()->FindBin(firsty);
  Int_t lastybin  = hist2->GetYaxis()->FindBin(lasty);
  TH1D* histd = hist2->ProjectionX(name.c_str(), firstybin, lastybin, option);

  TList* list = gDirectory->GetList();
  if(list){
    histd->SetDirectory(0);
    list->AddAfter(hist2, histd);
  }

  TArtCanvasManager::cdnext();
  TArtHistManager::DrawHist(histd);

  return histd;
}

TH1D* TArtSimpleFunction::BanX(double firstx, double lastx, Option_t* option)
{
  TH1* hist = gTH1;
  if(!hist) return 0;
  if(!hist->InheritsFrom("TH2")){
    std::cout << " not 2d hist" << std::endl;
    return 0;
  }

  TH2* hist2 = (TH2*)hist;
  std::string name(hist2->GetName());
  name += "_bnx";
  Int_t firstxbin = hist2->GetXaxis()->FindBin(firstx);
  Int_t lastxbin  = hist2->GetXaxis()->FindBin(lastx);
  TH1D* histd = hist2->ProjectionY(name.c_str(), firstxbin, lastxbin, option);

  TList* list = gDirectory->GetList();
  if(list){
    histd->SetDirectory(0);
    list->AddAfter(hist2, histd);
  }

  TArtCanvasManager::cdnext();
  TArtHistManager::DrawHist(histd);

  return histd;
}
