#ifndef _TARTBEAM_H_
#define _TARTBEAM_H_

#include <TObjArray.h>
#include <TNamed.h>

class TArtBigRIPSParameters;
class TArtRIPSArray;
class TArtRIPS;
class TArtTOFArray;
class TArtTOF;
class TArtICArray;
class TArtIC;
class TArtBeamPara;

class TArtBeamArray : public TObjArray {

 public:
  TArtBeamArray() :loadmap(false){}
  TArtBeamArray(TArtBigRIPSParameters *para) :loadmap(false){LoadSetup(para);}
  virtual ~TArtBeamArray(){}

  virtual void LoadSetup(TArtBigRIPSParameters *para);
  virtual void LoadMap(TArtRIPSArray *rips_array, TArtTOFArray *tof_array, TArtICArray *ic_array);
  virtual void ClearData();
  virtual void RecoAoQZ(); // reconstruct delta

  virtual Bool_t IsMapLoaded(){return loadmap;}

 private:
  Bool_t loadmap;

  ClassDef(TArtBeamArray,1);

};

class TArtBeam : public TNamed
{
 public: 

  TArtBeam(TArtBeamPara *para);
  virtual ~TArtBeam();

  virtual void RecoAoQZ();
  virtual void ClearData();
  virtual void LoadMap(TArtRIPSArray *rips_array, TArtTOFArray *tof_array, TArtICArray *ic_array);

  void     SetAoQ(Double_t b){aoq = b;}
  Double_t GetAoQ(){return aoq;}
  void     SetBeta(Double_t b){beta = b;}
  Double_t GetBeta(){return beta;}
  void     SetZet(Double_t b){zet = b;}
  Double_t GetZet(){return zet;}

  // calibration parameter
  Int_t        GetID();
  TArtRIPS * GetForwardRIPS(){return forward_rips;}
  TArtRIPS * GetBackwardRIPS(){return backward_rips;}

 private:

  // reconstructed aoq
  Double_t aoq;

  // reconstructed beta
  Double_t zet;

  // reconstructed beta
  Double_t beta;

  Double_t  clight;
  Double_t  mnucleon;

  // setup
  TArtBeamPara * para;
  TArtRIPS * forward_rips;
  TArtRIPS * backward_rips;
  TArtTOF  * tof;
  TArtIC   * ic;

  ClassDef(TArtBeam,1);

};

#endif
