#include "TArtIC.h" 
#include "TArtICPara.h" 
#include "TArtCore.h" 
#include "TArtMapper.h" 
#include "TArtBigRIPSParameters.h"

#include <TRandom.h>
#include <TList.h>
#include <TMath.h>

//__________________________________________________________
void TArtICArray::LoadSetup(TArtBigRIPSParameters *para) 
{

  Clear();
  TList *listOfICPara = (TList *)para->GetListOfICPara();
  TIter next(listOfICPara);
  TArtICPara *p;
  while(p = (TArtICPara*)next()) {
    Add(new TArtIC(p));
  }

}

//__________________________________________________________
void TArtICArray::LoadRawDataMap(TArtMapper *mapper) 
{
  for(int i=0;i<GetEntries();i++)
    ((TArtIC*)At(i))->LoadRawDataMap(mapper);
}

//__________________________________________________________
void TArtICArray::ClearData()
{
  for(int i=0;i<GetEntries();i++)
    ((TArtIC*)At(i))->ClearData();
}

//__________________________________________________________
void TArtICArray::ReconstructData()
{
  for(int i=0;i<GetEntries();i++)
    ((TArtIC*)At(i))->ReconstructData();
}


//__________________________________________________________
TArtIC::TArtIC(TArtICPara *p)
  :fReconstructed(false), para(p)
{
}

//__________________________________________________________
TArtIC::~TArtIC()  {
}

//__________________________________________________________
void TArtIC::LoadRawDataMap(TArtMapper *mapper) {
  for(int i=0;i<NUM_IC_CHANNEL;i++){
    TArtCore::Info(__FILE__,"mapping: ch%d of %s is %s", i, GetDetectorName()->Data(), ((TArtRIDFMap *)para->GetMap(i))->GetMapInfo());
    mapper->LoadRecoConfig(para->GetMap(i), &fChEnergyRaw[i]);
  }
}

//__________________________________________________________
void TArtIC::ClearData() {

  // zero all the data:
  for(int i=0;i<NUM_IC_CHANNEL;i++)  
    fChEnergyRaw[i] = 0;
  fFired = false;
  fReconstructed = false;

}

//__________________________________________________________
void TArtIC::ReconstructData()  {

  static TRandom *r = new TRandom();
  double fAvSum = 0, fSqSum = 1;
  double fNumberOfChannelsFired = 0.0;

  fCalMeVSqSum   = 0.0;
  fCalMeVAvSum   = 0.0;
  
  float random = r->Rndm();

  for(int i=0;i<NUM_IC_CHANNEL;i++) {
    if(fChEnergyRaw[i]>250) { //Set the threshold to higher values to avoid all the junk.
      random = r->Rndm();
      fFired = true; 
      fAvSum = fAvSum+ fChEnergyRaw[i]-0.5 + random;
      fSqSum = fSqSum*(fChEnergyRaw[i]-0.5 + random);
      fNumberOfChannelsFired ++;
    }
  }

  if(fFired) {
    fAvSum         = fAvSum/fNumberOfChannelsFired;
    fSqSum         = TMath::Power(fSqSum,1/fNumberOfChannelsFired);
    fCalMeVAvSum   = para->GetCh2MeV(0) + para->GetCh2MeV(1)*fAvSum;
    fCalMeVSqSum   = para->GetCh2MeV(0) + para->GetCh2MeV(1)*fSqSum;
  }

  fReconstructed = true;

}

//__________________________________________________________
double TArtIC::RecoZet(double beta)  {

  double ionpair = para->GetIonPair();

  double de_v = TMath::Log(ionpair*beta*beta) - TMath::Log((1-beta*beta)) - beta*beta;
  return para->GetZCoef(0)*TMath::Sqrt(fCalMeVSqSum/de_v)*beta + para->GetZCoef(1);


}

//__________________________________________________________
Int_t TArtIC::GetID() {
  return para->GetID();
}
//__________________________________________________________
Int_t TArtIC::GetFpl() {
  return para->GetFpl();
}
//__________________________________________________________
TString * TArtIC::GetDetectorName() {
  return (TString *)para->GetDetectorName();
}
