#include "TArtPlastic.h" 
#include "TArtPlasticPara.h" 
#include "TArtCore.h" 
#include "TArtMapper.h" 
#include "TArtBigRIPSParameters.h"

#include <TRandom.h>
#include <TList.h>

//__________________________________________________________
void TArtPlasticArray::LoadSetup(TArtBigRIPSParameters *para) 
{

  Clear();
  TList *listOfPlasticPara = (TList *)para->GetListOfPlasticPara();
  TIter next(listOfPlasticPara);
  TArtPlasticPara *p;
  while(p = (TArtPlasticPara*)next()) {
    Add(new TArtPlastic(p));
  }

}

//__________________________________________________________
void TArtPlasticArray::LoadRawDataMap(TArtMapper *mapper) 
{
  TArtCore::Info(__FILE__,"LoadRawDataMap");
  for(int i=0;i<GetEntries();i++)
    ((TArtPlastic*)At(i))->LoadRawDataMap(mapper);
}

//__________________________________________________________
void TArtPlasticArray::ClearData()
{
  for(int i=0;i<GetEntries();i++)
    ((TArtPlastic*)At(i))->ClearData();
}

//__________________________________________________________
void TArtPlasticArray::ReconstructData()
{
  for(int i=0;i<GetEntries();i++)
    ((TArtPlastic*)At(i))->ReconstructData();
}

//__________________________________________________________
TArtPlastic::TArtPlastic(TArtPlasticPara *p)
  :fReconstructed(false), zposition(0), para(p)
{
}

//__________________________________________________________
TArtPlastic::~TArtPlastic()  {
}

//__________________________________________________________
void TArtPlastic::LoadRawDataMap(TArtMapper *mapper) {

  TArtCore::Info(__FILE__,"mapping: T Left of %s is %s", GetDetectorName()->Data(), ((TArtRIDFMap *)para->GetTLMap())->GetMapInfo());
  mapper->LoadRecoConfig(para->GetTLMap(), &fTLRaw);
  TArtCore::Info(__FILE__,"mapping: T Right of %s is %s", GetDetectorName()->Data(), ((TArtRIDFMap *)para->GetTRMap())->GetMapInfo());
  mapper->LoadRecoConfig(para->GetTRMap(), &fTRRaw);
  TArtCore::Info(__FILE__,"mapping: Q Left of %s is %s", GetDetectorName()->Data(), ((TArtRIDFMap *)para->GetQLMap())->GetMapInfo());
  mapper->LoadRecoConfig(para->GetQLMap(), &fQLRaw);
  TArtCore::Info(__FILE__,"mapping: Q Right of %s is %s", GetDetectorName()->Data(), ((TArtRIDFMap *)para->GetQRMap())->GetMapInfo());
  mapper->LoadRecoConfig(para->GetQRMap(), &fQRRaw);

}

//__________________________________________________________
void TArtPlastic::ClearData() {

  fTLRaw  = 0;  fTRRaw  = 0;
  fQLRaw  = 0;  fQRRaw  = 0;
  fTime   = 0;
  fFired  = false;
  fReconstructed = false;

}

//__________________________________________________________
void TArtPlastic::ReconstructData()  {

  static TRandom *r = new TRandom();
  Double_t fTimeL = -1000., fTimeR = -1000.;
  Bool_t fLFired = false, fRFired = false;

  float random = r->Rndm();
  if(fTLRaw>0 && fTLRaw<4000) {
    fLFired = true;  
    fTimeL = (fTLRaw-0.5 + random) * para->GetTCalLeft();
  }
  random = r->Rndm();
  if(fTRRaw>0 && fTRRaw<4000) {
    fRFired = true;
    fTimeR = (fTRRaw-0.5 + random) * para->GetTCalRight();
  }
  if(fLFired && fRFired) fFired = true;
  
  if(fFired) {
    fTime = (fTimeL+fTimeR)/2.;
    //TArtCore::Info(__FILE__,"ch %d: fpl: %d, tlraw: %d, trraw: %d, tl: %f, tr: %f, t:%f", i , fFocalPlane, fTLRaw,fTRRaw,(float)fTL,(float)fTR,(float)fTOF);
    //fTDiff = fTL-fTR;
    //fX     = fDT2X[0] * pow(fTDiff,2) + fDT2X[1] * fTDiff + fDT2X[2]; 
  }
  
  fReconstructed = true;

}

//__________________________________________________________
Int_t TArtPlastic::GetID() {
  return para->GetID();
}
//__________________________________________________________
TString * TArtPlastic::GetDetectorName() {
  return (TString *)para->GetDetectorName();
}
//__________________________________________________________
Int_t TArtPlastic::GetFpl() {
  return para->GetFpl();
}
//__________________________________________________________
Double_t TArtPlastic::GetZoffset() {
  return para->GetZoffset();
}
