#include "TArtTOF.h"
#include "TArtTOFPara.h"
#include "TArtCore.h"
#include "TArtPlastic.h"
#include "TArtBigRIPSParameters.h"

#include <TList.h>
#include <TMath.h>

void TArtTOFArray::LoadSetup(TArtBigRIPSParameters *para) 
{

  Clear();
  TList *listOfTOFPara = (TList *)para->GetListOfTOFPara();
  TIter next(listOfTOFPara);
  TArtTOFPara *p;
  while(p = (TArtTOFPara*)next()) {
    Add(new TArtTOF(p));
  }

}

//__________________________________________________________
void TArtTOFArray::LoadPlasticMap(TArtPlasticArray *pls) 
{
  for(int i=0;i<GetEntries();i++)
    ((TArtTOF*)At(i))->LoadPlasticMap(pls);
  loadplmap = true;
}

//__________________________________________________________
void TArtTOFArray::ClearData()
{
  for(int i=0;i<GetEntries();i++)
    ((TArtTOF*)At(i))->ClearData();
}

//__________________________________________________________
void TArtTOFArray::RecoTOF()
{
  for(int i=0;i<GetEntries();i++)
    ((TArtTOF*)At(i))->RecoTOF();
}

//__________________________________________________________
TArtTOF::TArtTOF(TArtTOFPara *p) : para(p)
{
  clight = 299.792458 ; // mm/nsec
}

//__________________________________________________________
TArtTOF::~TArtTOF() 
{
}

//__________________________________________________________
void TArtTOF::LoadPlasticMap(TArtPlasticArray *pls) 
{
  forward_pla = NULL;
  backward_pla = NULL;

  TString * fplname = para->GetForwardPlName();
  TString * bplname = para->GetBackwardPlName();

  for(int i=0;i<pls->GetEntries();i++){
    TArtPlastic * pl = (TArtPlastic *)pls->At(i);

    if( strcmp(pl->GetDetectorName()->Data(), fplname->Data() ) == 0 ){
      if( NULL == forward_pla )
	forward_pla = pl;
      else
	TArtCore::Info(__FILE__,"multiple plastic is defined!: %s",fplname->Data());
    }

    if( strcmp(pl->GetDetectorName()->Data(), bplname->Data() ) == 0 ){
      if( NULL == backward_pla )
	backward_pla = pl;
      else
	TArtCore::Info(__FILE__,"multiple plastic is defined!: %s",bplname->Data());
    }
      
  }

  if(forward_pla&&backward_pla){
    Double_t f_zpos = forward_pla->GetZpos();
    Double_t b_zpos = backward_pla->GetZpos();
    if(f_zpos>0&&b_zpos>0){
      tof_length = b_zpos - f_zpos;
      TArtCore::Info(__FILE__,"tof length for: %s - %s is %f",fplname->Data(), bplname->Data(), (float)tof_length);
    }
  }
  else
    TArtCore::Info(__FILE__,"tof length can not be defined for: %s - %s",fplname->Data(), bplname->Data());

}

//__________________________________________________________
void TArtTOF::ClearData()
{
  tof = 0; beta = 0; gamma = 0;
}

//__________________________________________________________
void TArtTOF::RecoTOF()
{

  tof = backward_pla->GetTime() - forward_pla->GetTime() + para->GetOffset();
  beta = tof_length/tof/clight;
  gamma = 1/TMath::Sqrt(1-beta*beta);

}

//__________________________________________________________
Int_t TArtTOF::GetID() {
  return para->GetID();
}

//__________________________________________________________
TString * TArtTOF::GetForwardPlName() {
  return para->GetForwardPlName();
}

//__________________________________________________________
TString * TArtTOF::GetBackwardPlName() {
  return para->GetBackwardPlName();
}

